@@ -2,7 +2,10 @@ import logging
 from itertools import izip
 from operator import itemgetter
 
-from pyes import ES, VERSION
+from pyes import ES
+from pyes import VERSION as PYES_VERSION
+
+from elasticutils._version import __version__
 
 
 log = logging.getLogger('elasticutils')
@@ -10,8 +13,8 @@ log = logging.getLogger('elasticutils')
 
 DEFAULT_HOSTS = ['localhost:9200']
 DEFAULT_TIMEOUT = 5
-DEFAULT_DOCTYPES = None
-DEFAULT_INDEXES = 'default'
+DEFAULT_DOCTYPES = ['document']
+DEFAULT_INDEXES = ['default']
 DEFAULT_DUMP_CURL = None
 
 
@@ -36,15 +39,23 @@ def get_es(hosts=None, default_indexes=None, timeout=None, dump_curl=None,
 
     Examples:
 
-    >>> get_es()
-    >>> get_es(hosts=['localhost:9200'])
-    >>> get_es(timeout=30)  # good for indexing
-    >>> get_es(default_indexes=['sumo_prod_20120627']
+    >>> es = get_es()
+
+
+    >>> es = get_es(hosts=['localhost:9200'])
+
+
+    >>> es = get_es(timeout=30)  # good for indexing
+
+
+    >>> es = get_es(default_indexes=['sumo_prod_20120627']
+
+
     >>> class CurlDumper(object):
     ...     def write(self, text):
     ...         print text
     ...
-    >>> get_es(dump_curl=CurlDumper())
+    >>> es = get_es(dump_curl=CurlDumper())
 
     """
     # Cheap way of de-None-ifying things
@@ -67,7 +78,7 @@ def get_es(hosts=None, default_indexes=None, timeout=None, dump_curl=None,
     # is set it manually after the ES has been created and
     # defaults['dump_curl'] is truthy. This might not work for all
     # values of dump_curl.
-    if VERSION[0:2] == (0, 15) and dump_curl is not None:
+    if PYES_VERSION[0:2] == (0, 15) and dump_curl is not None:
         es.dump_curl = dump_curl
 
     return es
@@ -192,6 +203,7 @@ def _boosted_value(name, action, key, value, boost):
 # Maps ElasticUtils field actions to their ElasticSearch query names.
 ACTION_MAP = {
     None: 'term',  # Default to term
+    'in': 'in',
     'term': 'term',
     'startswith': 'prefix',  # Backwards compatability
     'prefix': 'prefix',
@@ -201,9 +213,18 @@ ACTION_MAP = {
 
 
 class S(object):
-    """
-    Represents a lazy ElasticSearch lookup, with a similar api to
-    Django's QuerySet.
+    """Represents a lazy ElasticSearch Search API request.
+
+    The API for `S` takes inspiration from Django's QuerySet.
+
+    `S` can be either typed or untyped. An untyped `S` returns dict
+    results by default.
+
+    An `S` is lazy in the sense that it doesn't do an ElasticSearch
+    search request until it's forced to evaluate by either iterating
+    over it, calling ``.count``, doing ``len(s)``, or calling
+    ``.facet_count``.
+
     """
     def __init__(self, type_=None):
         """Create and return an S.
@@ -242,7 +263,7 @@ class S(object):
         the search.
 
         :arg settings: the settings you'd use to build the ES---same
-            as what you'd pass to :fun:`get_es`.
+            as what you'd pass to :py:func:`get_es`.
 
         """
         return self._clone(next_step=('es', settings))
@@ -340,6 +361,12 @@ class S(object):
         new.field_boosts.update(kw)
         return new
 
+    def demote(self, amount_, **kw):
+        """
+        Returns a new S instance with boosting query and demotion.
+        """
+        return self._clone(next_step=('demote', (amount_, kw)))
+
     def facet(self, *args, **kw):
         """
         Return a new S instance with facet args combined with existing
@@ -429,6 +456,7 @@ class S(object):
         fields = set(['id'])
         facets = {}
         facets_raw = {}
+        demote = None
         highlight_fields = set()
         highlight_options = {}
         explain = False
@@ -454,6 +482,8 @@ class S(object):
                 explain = value
             elif action == 'query':
                 queries.extend(self._process_queries(value))
+            elif action == 'demote':
+                demote = (value[0], self._process_queries(value[1]))
             elif action == 'filter':
                 filters.extend(_process_filters(value))
             elif action == 'facet':
@@ -486,6 +516,15 @@ class S(object):
         elif queries:
             qs['query'] = queries[0]
 
+        if demote is not None:
+            qs['query'] = {
+                'boosting': {
+                    'positive': qs['query'],
+                    'negative': demote[1],
+                    'negative_boost': demote[0]
+                    }
+                }
+
         if fields:
             qs['fields'] = list(fields)
 
@@ -560,6 +599,10 @@ class S(object):
                     {'range': {field_name: _boosted_value(
                                 field_name, field_action, key, val, boost)}})
 
+            else:
+                raise InvalidFieldActionError(
+                    '%s is not a valid field action' % field_action)
+
         if or_:
             rv.append({'bool': {'should': self._process_queries(or_.items())}})
         return rv
@@ -595,7 +638,6 @@ class S(object):
         for action, value in reversed(self.steps):
             if action == 'indexes':
                 return value
-
         return default_indexes
 
     def get_doctypes(self, default_doctypes=DEFAULT_DOCTYPES):
@@ -635,6 +677,127 @@ class S(object):
         return facets
 
 
+class MLT(object):
+    """Represents a lazy ElasticSearch More Like This API request.
+
+    This is lazy in the sense that it doesn't evaluate and execute the
+    ElasticSearch request unless you force it to by iterating over it
+    or getting the length of the search results.
+
+    For example::
+
+    >>> mlt = MLT(2034, index='addons_index', doctype='addon')
+    >>> num_related_documents = len(mlt)
+    >>> num_related_documents = list(mlt)
+
+    """
+    def __init__(self, id_, s=None, fields=None, index=None, doctype=None,
+                 es=None, **query_params):
+        """
+        When the MLT is evaluated, it generates a list of dict results.
+
+        :arg id_: The id of the document we want to find more like.
+        :arg s: An instance of an S. The query is passed in the body of
+            the more like this request.
+        :arg fields: A list of fields to use for more like this.
+        :arg index: The index to use. Falls back to the first index
+            listed in s.
+        :arg doctype: The doctype to use. Falls back to the first
+            doctype listed in s.
+        :arg es: The ES object to use. If you don't provide one, then it
+            will create one for you.
+        :arg query_params: Any additional query parameters for the
+            more like this call.
+
+        .. Note::
+
+           You must specify either an `s` or the `index` and `doctype`
+           arguments. Omitting them will result in a `ValueError`.
+
+        """
+        # You have to provide either an s OR an index and a doc_type.
+        if s is None and (index is None or doctype is None):
+            raise ValueError(
+                'Either you must provide a valid s or index and doc_type')
+
+        self.s = s
+        if s is not None:
+            # If an index or doctype isn't given, we use the first one
+            # in the S.
+            self.index = index or s.get_indexes()[0]
+            self.doctype = doctype or s.get_doctypes()[0]
+            self.type = s.type
+        else:
+            self.index = index
+            self.doctype = doctype
+            self.type = None
+
+        self.id = id_
+        self.fields = fields
+        self.es = es
+        self.query_params = query_params
+        self._results_cache = None
+
+    def __iter__(self):
+        return iter(self._do_search())
+
+    def __len__(self):
+        return len(self._do_search())
+
+    def get_es(self):
+        """Returns an ES
+
+        * If there's an s, then it returns that ES.
+        * If the es was provided in the constructor, then it returns that ES.
+        * Otherwise, it creates a new ES and returns that.
+
+        Override this if that behavior isn't correct for you.
+
+        """
+        if self.s:
+            return self.s.get_es()
+
+        return self.es or get_es()
+
+    def raw(self):
+        """
+        Build query and passes to ElasticSearch, then returns the raw
+        format returned.
+        """
+        es = self.get_es()
+
+        kwargs = {}
+        path = es._make_path([self.index, self.doctype, self.id, '_mlt'])
+
+        params = dict(self.query_params)
+
+        if self.fields and 'mlt_fields' not in params:
+            params['mlt_fields'] = ','.join(self.fields)
+        kwargs['params'] = params
+
+        if self.s:
+            kwargs['body'] = self.s._build_query()
+
+        try:
+            hits = es._send_request('GET', path, **kwargs)
+            log.debug(hits)
+        except Exception:
+            log.error(kwargs)
+            raise
+        log.debug('[%s] %s' % (hits['took'], kwargs))
+        return hits
+
+    def _do_search(self):
+        """
+        Perform the mlt call, then convert that raw format into a
+        SearchResults instance and return it.
+        """
+        if not self._results_cache:
+            hits = self.raw()
+            self._results_cache = DictSearchResults(self.type, hits, None)
+        return self._results_cache
+
+
 class SearchResults(object):
     def __init__(self, type, results, fields):
         self.type = type
